import { Pipe, PipeTransform } from '@angular/core';
import { BinInputVariant } from '@pv-frontend/pv-shared-services/common';

@Pipe({
  name: 'pvBinFormat',
})
export class BinFormatPipe implements PipeTransform {
  public transform(
    value?: string,
    format: BinInputVariant = 'first4',
    separator: string = '_',
    replacementChar: string = 'X'
  ): string {
    const bin = value?.replace(separator, '');
    if (!bin) {
      return '';
    }

    const getSubString = (slice: number[]): string => bin.slice(...slice);

    const formatTransformations = {
      first4: `${getSubString([0, 4])} ${replacementChar.repeat(
        4
      )} ${replacementChar.repeat(4)} ${replacementChar.repeat(4)}`,
      first4_last4: `${getSubString([0, 4])} ${replacementChar.repeat(
        4
      )} ${replacementChar.repeat(4)} ${getSubString([bin?.length - 4])}`,
      first6: `${getSubString([0, 4])} ${getSubString([
        bin?.length - 2,
      ])}${replacementChar.repeat(2)} ${replacementChar.repeat(
        4
      )} ${replacementChar.repeat(4)}`,
      first6_last4: `${getSubString([0, 4])} ${getSubString([
        bin.length - 2,
      ])}${replacementChar.repeat(2)} ${replacementChar.repeat(
        4
      )} ${getSubString([bin.length - 4])}`,
      last4: `${replacementChar.repeat(4)} ${replacementChar.repeat(
        4
      )} ${replacementChar.repeat(4)} ${getSubString([bin.length - 4])}`,
      first8: `${getSubString([0, 4])} ${getSubString([
        bin.length - 4,
      ])} ${replacementChar.repeat(4)} ${replacementChar.repeat(4)}`,
      first9: `${getSubString([0, 4])} ${getSubString([
        bin.length - 4,
      ])} ${bin.substring(bin.length - 1)}${replacementChar.repeat(
        3
      )} ${replacementChar.repeat(4)}`,
      last8: `${replacementChar.repeat(4)} ${replacementChar.repeat(
        4
      )} ${getSubString([bin?.length - 8])}`,
    };

    return formatTransformations[format] || bin;
  }
}
